==> 1.14.514.131-1.21.4.tar.gz: serverplayeronlinetracker/src/main/java/com/example/playertime/PlayerTimeMod.java <==
package com.example.playertime;

import com.mojang.brigadier.arguments.IntegerArgumentType;
import net.fabricmc.api.ModInitializer;
import net.fabricmc.fabric.api.command.v2.CommandRegistrationCallback;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerLifecycleEvents;
import net.fabricmc.fabric.api.networking.v1.ServerPlayConnectionEvents;
import net.fabricmc.loader.api.FabricLoader;
import net.minecraft.server.command.CommandManager;
import net.minecraft.server.command.ServerCommandSource;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.text.ClickEvent;
import net.minecraft.text.MutableText;
import net.minecraft.text.Text;
import net.minecraft.util.Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

public class PlayerTimeMod implements ModInitializer {
    public static final Logger LOGGER = LoggerFactory.getLogger("PlayerTimeTracker");
    private static PlayerTimeTracker timeTracker;
    private static WebServer webServer;
    private static ModConfig config;
    public static LocalizationManager localizationManager; // 新增本地化管理器

    // TODO 定时保存配置文件没整，暂时硬编码
    private ScheduledExecutorService scheduler;
    private ScheduledFuture<?> saveTask;
    private static long AUTO_SAVE_INTERVAL_SECONDS;

    @Override
    public void onInitialize() {

        config = new ModConfig(FabricLoader.getInstance().getConfigDir());
        localizationManager = new LocalizationManager(config.getLanguage());

        scheduler = Executors.newSingleThreadScheduledExecutor();


        try {
            LOGGER.info("[在线时间] 初始化 玩家在线时长视奸Mod");

            AUTO_SAVE_INTERVAL_SECONDS = config.getSeconds();

            ServerLifecycleEvents.SERVER_STARTING.register(server -> {
                timeTracker = new PlayerTimeTracker(server);
                try {
                    webServer = new WebServer(timeTracker, config.getWebPort(), server); // 传入 MinecraftServer
                    webServer.start();
                    LOGGER.info("[在线时间] Web服务器在端口 " + config.getWebPort() + "启动");
                } catch (Exception e) {
                    LOGGER.error("[在线时间] 无法启动Web服务器", e);
                }

                LOGGER.info("[在线时间] 每{}秒（{}分钟）安排自动保存任务。",
                        AUTO_SAVE_INTERVAL_SECONDS, AUTO_SAVE_INTERVAL_SECONDS / 60);
                saveTask = scheduler.scheduleAtFixedRate(
                        () -> {
                            if (timeTracker != null) {
                                LOGGER.info("[在线时间] 自动保存玩家数据中...");
                                timeTracker.saveAll();
                                LOGGER.info("[在线时间] 自动保存完成。");
                            }
                        },
                        AUTO_SAVE_INTERVAL_SECONDS,
                        AUTO_SAVE_INTERVAL_SECONDS,
                        TimeUnit.SECONDS 
                );
            });

            ServerPlayConnectionEvents.JOIN.register((handler, sender, server) -> {
                if (timeTracker != null) {
                    timeTracker.onPlayerJoin(handler.player);
                }
            });

            ServerPlayConnectionEvents.DISCONNECT.register((handler, server) -> {
                if (timeTracker != null) {
                    timeTracker.onPlayerLeave(handler.player);
                }
            });

            ServerLifecycleEvents.SERVER_STOPPING.register(server -> {
                LOGGER.info("[在线时间] 服务器停止 - 正在保存数据");

                if (saveTask != null) {
                    saveTask.cancel(false);
                    LOGGER.info("[在线时间] 自动保存任务已取消。");
                }
                if (scheduler != null && !scheduler.isShutdown()) {
                    scheduler.shutdown();
                    try {
                        if (!scheduler.awaitTermination(5, TimeUnit.SECONDS)) {
                            LOGGER.warn("[在线时间] 调度程序未在5秒内终止。");
                        }
                    } catch (InterruptedException e) {
                        LOGGER.error("[在线时间] 调度程序终止被中断", e);
                        Thread.currentThread().interrupt();
                    }
                    LOGGER.info("[在线时间] 调度程序关闭");
                }


                if (webServer != null) {
                    webServer.stop();
                }
                if (timeTracker != null) {
                    for (ServerPlayerEntity player : server.getPlayerManager().getPlayerList()) {
                        timeTracker.onPlayerLeave(player);
                    }
                    timeTracker.saveAll();
                }
            });


        } catch (Exception e) {
            LOGGER.error("[在线时间] Mod出屎化失败", e);
            if (scheduler != null && !scheduler.isShutdown()) {
                scheduler.shutdownNow();
            }
            throw new RuntimeException("[在线时间] Mod初始化失败 ", e);
        }
        registerCommands();
    }

    public static ModConfig getConfig() {
        return config;
    }

    public static PlayerTimeTracker getTimeTracker() {
        return timeTracker;
    }

    public static LocalizationManager getLocalizationManager() {
        return localizationManager;
    }


    public static void registerCommands() {
        CommandRegistrationCallback.EVENT.register((dispatcher, registryAccess, environment) -> {
            dispatcher.register(CommandManager.literal("onlineTime")
                    .requires(source -> source.hasPermissionLevel(0))
                    .executes(context -> showOnlineTime(context.getSource(), 1)) // 默认第一页
                    .then(CommandManager.argument("page", IntegerArgumentType.integer(1))
                            .executes(context -> showOnlineTime(
                                    context.getSource(),
                                    IntegerArgumentType.getInteger(context, "page")
                            ))
                    )
            );
        });
    }

    private static int showOnlineTime(ServerCommandSource source, int requestedPage) {
        ServerPlayerEntity player = source.getPlayer();
        if (player == null) return 0;

        CompletableFuture.runAsync(() -> {
            PlayerTimeTracker tracker = getTimeTracker();
            if (tracker != null) {
                Map<String, String> stats = tracker.getWhitelistedPlayerStats();

                List<String> sorted = stats.entrySet().stream()
                        .sorted((a, b) -> comparePlayTime(b.getValue(), a.getValue()))
                        .map(entry -> formatPlayerTime(entry.getKey(), entry.getValue()))
                        .collect(Collectors.toList());

                sendPaginatedMessage(player, sorted, requestedPage);
            }
        }, Util.getMainWorkerExecutor());

        return 1;
    }

    private static int comparePlayTime(String a, String b) {
        String timeA = a.substring(a.indexOf(':') + 1).trim().split(" \\| ")[0];
        String timeB = b.substring(b.indexOf(':') + 1).trim().split(" \\| ")[0];

        return Long.compare(parseTimeToSeconds(timeB), parseTimeToSeconds(timeA));
    }

    private static long parseTimeToSeconds(String timeStr) {
        long seconds = 0;
        String[] parts = timeStr.split(" ");
        for (String part : parts) {
            if (part.endsWith("h")) {
                try {
                    seconds += Integer.parseInt(part.replace("h", "")) * 3600;
                } catch (NumberFormatException ignored) {}
            } else if (part.endsWith("m")) {
                try {
                    seconds += Integer.parseInt(part.replace("m", "")) * 60;
                } catch (NumberFormatException ignored) {}
            }
        }
        return seconds;
    }

    private static String formatPlayerTime(String name, String timeStr) {
        return String.format("§e%s§r: %s", name, timeStr);
    }

    private static void sendPaginatedMessage(ServerPlayerEntity player, List<String> lines, int page) {
        int pageSize = 10;
        int totalPages = (lines.size() + pageSize - 1) / pageSize;
        page = Math.max(1, Math.min(page, totalPages));

        int from = (page - 1) * pageSize;
        int to = Math.min(from + pageSize, lines.size());

        player.sendMessage(Text.literal(localizationManager.getString("playertime.command.title", page, totalPages)), false);

        for (int i = from; i < to; i++) {
            player.sendMessage(Text.literal(lines.get(i)), false);
        }

        MutableText footer = Text.literal("");
        if (page > 1) {
            int finalPage = page;
            footer.append(Text.literal(localizationManager.getString("playertime.command.prev_page"))
                    .styled(style -> style.withClickEvent(new ClickEvent(
                            ClickEvent.Action.RUN_COMMAND,
                            "/onlineTime " + (finalPage - 1)
                    )))
                    .append(" "));
        }

        footer.append(Text.literal(localizationManager.getString("playertime.command.total_players", lines.size())));

        if (page < totalPages) {
            int finalPage1 = page;
            footer.append(" ").append(Text.literal(localizationManager.getString("playertime.command.next_page"))
                    .styled(style -> style.withClickEvent(new ClickEvent(
                            ClickEvent.Action.RUN_COMMAND,
                            "/onlineTime " + (finalPage1 + 1)
                    ))));
        }

        player.sendMessage(footer, false);
    }

}

==> 1.14.514.132.tar.gz: serverplayeronlinetracker/src/main/java/com/example/playertime/PlayerTimeMod.java <==
package com.example.playertime;

import com.mojang.brigadier.arguments.IntegerArgumentType;
import net.fabricmc.api.ModInitializer;
import net.fabricmc.fabric.api.command.v2.CommandRegistrationCallback;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerLifecycleEvents;
import net.fabricmc.fabric.api.networking.v1.ServerPlayConnectionEvents;
import net.fabricmc.loader.api.FabricLoader;
import net.minecraft.server.command.CommandManager;
import net.minecraft.server.command.ServerCommandSource;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.text.ClickEvent;
import net.minecraft.text.MutableText;
import net.minecraft.text.Text;
import net.minecraft.util.Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;

public class PlayerTimeMod implements ModInitializer {
    public static final Logger LOGGER = LoggerFactory.getLogger("PlayerTimeTracker");
    private static PlayerTimeTracker timeTracker;
    private static WebServer webServer;
    private static ModConfig config;
    public static LocalizationManager localizationManager;

    @Override
    public void onInitialize() {
        config = new ModConfig(FabricLoader.getInstance().getConfigDir());
        localizationManager = new LocalizationManager(config.getLanguage());

        try {
            LOGGER.info("[在线时间] 初始化 玩家在线时长视奸Mod");

            ServerLifecycleEvents.SERVER_STARTING.register(server -> {
                timeTracker = new PlayerTimeTracker(server);
                try {
                    webServer = new WebServer(timeTracker, config.getWebPort(), server);
                    webServer.start();
                    LOGGER.info("[在线时间] Web服务器在端口 " + config.getWebPort() + "启动");
                } catch (Exception e) {
                    LOGGER.error("[在线时间] 无法启动Web服务器", e);
                }
            });

            ServerPlayConnectionEvents.JOIN.register((handler, sender, server) -> {
                if (timeTracker != null) {
                    timeTracker.onPlayerJoin(handler.player);
                }
            });

            ServerPlayConnectionEvents.DISCONNECT.register((handler, server) -> {
                if (timeTracker != null) {
                    timeTracker.onPlayerLeave(handler.player);
                }
            });

            ServerLifecycleEvents.SERVER_STOPPING.register(server -> {
                LOGGER.info("[在线时间] 服务器停止 - 正在保存数据");

                if (webServer != null) {
                    webServer.stop();
                }
                if (timeTracker != null) {
                    for (ServerPlayerEntity player : server.getPlayerManager().getPlayerList()) {
                        timeTracker.onPlayerLeave(player);
                    }
                    timeTracker.saveAll();
                }
            });

        } catch (Exception e) {
            LOGGER.error("[在线时间] Mod初始化失败", e);
            throw new RuntimeException("[在线时间] Mod初始化失败 ", e);
        }
        registerCommands();
    }

    public static ModConfig getConfig() {
        return config;
    }

    public static PlayerTimeTracker getTimeTracker() {
        return timeTracker;
    }

    public static LocalizationManager getLocalizationManager() {
        return localizationManager;
    }

    public static void registerCommands() {
        CommandRegistrationCallback.EVENT.register((dispatcher, registryAccess, environment) -> {
            dispatcher.register(CommandManager.literal("onlineTime")
                    .requires(source -> source.hasPermissionLevel(0))
                    .executes(context -> showOnlineTime(context.getSource(), 1))
                    .then(CommandManager.argument("page", IntegerArgumentType.integer(1))
                            .executes(context -> showOnlineTime(
                                    context.getSource(),
                                    IntegerArgumentType.getInteger(context, "page")
                            ))
                    )
            );
        });
    }

    private static int showOnlineTime(ServerCommandSource source, int requestedPage) {
        ServerPlayerEntity player = source.getPlayer();
        if (player == null) return 0;

        CompletableFuture.runAsync(() -> {
            PlayerTimeTracker tracker = getTimeTracker();
            if (tracker != null) {
                Map<String, String> stats = tracker.getWhitelistedPlayerStats();

                List<String> sorted = stats.entrySet().stream()
                        .sorted((a, b) -> comparePlayTime(a.getValue(), b.getValue()))
                        .map(entry -> formatPlayerTime(entry.getKey(), entry.getValue()))
                        .collect(Collectors.toList());

                sendPaginatedMessage(player, sorted, requestedPage);
            }
        }, Util.getMainWorkerExecutor());

        return 1;
    }

    private static int comparePlayTime(String a, String b) {
        String timeA = a.substring(a.indexOf(':') + 1).trim().split(" \\| ")[0];
        String timeB = b.substring(b.indexOf(':') + 1).trim().split(" \\| ")[0];

        return Long.compare(parseTimeToSeconds(timeB), parseTimeToSeconds(timeA));
    }

    private static long parseTimeToSeconds(String timeStr) {
        long seconds = 0;
        String[] parts = timeStr.split(" ");
        for (String part : parts) {
            if (part.endsWith("h")) {
                try {
                    seconds += Integer.parseInt(part.replace("h", "")) * 3600;
                } catch (NumberFormatException ignored) {}
            } else if (part.endsWith("m")) {
                try {
                    seconds += Integer.parseInt(part.replace("m", "")) * 60;
                } catch (NumberFormatException ignored) {}
            }
        }
        return seconds;
    }

    private static String formatPlayerTime(String name, String timeStr) {
        return String.format("§e%s§r: %s", name, timeStr);
    }

    private static void sendPaginatedMessage(ServerPlayerEntity player, List<String> lines, int page) {
        int pageSize = 10;
        int totalPages = (lines.size() + pageSize - 1) / pageSize;
        page = Math.max(1, Math.min(page, totalPages));

        int from = (page - 1) * pageSize;
        int to = Math.min(from + pageSize, lines.size());

        player.sendMessage(Text.literal(localizationManager.getString("playertime.command.title", page, totalPages)), false);

        for (int i = from; i < to; i++) {
            player.sendMessage(Text.literal(lines.get(i)), false);
        }

        MutableText footer = Text.literal("");
        if (page > 1) {
            int finalPage = page;
            footer.append(Text.literal(localizationManager.getString("playertime.command.prev_page"))
                    .styled(style -> style.withClickEvent(new ClickEvent(
                            ClickEvent.Action.RUN_COMMAND,
                            "/onlineTime " + (finalPage - 1)
                    )))
                    .append(" "));
        }

        footer.append(Text.literal(localizationManager.getString("playertime.command.total_players", lines.size())));

        if (page < totalPages) {
            int finalPage1 = page;
            footer.append(" ").append(Text.literal(localizationManager.getString("playertime.command.next_page"))
                    .styled(style -> style.withClickEvent(new ClickEvent(
                            ClickEvent.Action.RUN_COMMAND,
                            "/onlineTime " + (finalPage1 + 1)
                    ))));
        }

        player.sendMessage(footer, false);
    }
}
